--------------------------------------------------------------------------------------------------------------------------
-- Lyric Catch 1.0.0.0
-- (C) 2011 あかつきみさき（みくちぃP）
-- 
-- このJob Pluginについて
--  ノートに入力された歌詞情報を取得し,テキストファイルに出力するVOCALOID3 Job Pluginです.
-- 
-- インストール
--  VOCALOID3上でパスを設定する必要があります.
--  適当なフォルダに置いて,VOCALOID3の「ジョブ→Jobプラグインを管理」から該当ディレクトリ(フォルダー)へパスを設定すれば完了です.
--  アンインストールはパスの削除,直接削除してください.
-- 
-- 動作環境
--  VOCALOID3 , VOCALOID3 Job Pluginが動作する環境
-- 
-- 使用方法
--  一つのトラック内の一つ以上のノートを含むパートを選択し,「ジョブ→Jobプラグインを実行」から実行してください.
--  ファイル名を入力してOKを押すと,vsqxファイルと同じディレクトリ(フォルダー)内に「vsqxファイル名.vsqx_入力ファイル名.txt」という名前のテキストファイルが生成されます.
--  デフォルトの場合は「vsqxファイル名.vsqx_パート名_VOCALOID3LyricData.txt」になっています.
--  複数のパートの同時書き出しには対応していません.一度に複数のパートで使用したい場合はパートを結合してください.
-- 
-- バージョン情報
--  2011/12/16 Ver1.0.0.0 Release
-- 
-- Q & A
--  Q1. パートを選択しているのにテキストファイルが出力されません.
--  A1. ノートを含んでいるか確認してください.選択されているパートとはMUSICAL EDITORでアクティブになっているパートのことです.
--------------------------------------------------------------------------------------------------------------------------

-- プラグインマニフェスト関数
function manifest()
    myManifest = {
        name = "Lyric Catch",
        comment = "ノートに入力された歌詞を取得します",
        author = "Misaki Akatsuki_102",
        pluginID = "{C77C1315-42B1-4e53-8959-8FF8241A2942}",
        pluginVersion = "1.0.0.0",
        apiVersion = "3.0.0.1"
    }

    return myManifest
end

-- VOCALOID3 Jobプラグインスクリプトのエントリポイント
function main(processParam, envParam)
    
-- 実行時に渡されたパラメータを取得
    local songPosTick = processParam.songPosTick
    local note = {}
    local noteEx = {}
    local noteExList = {}
    local noteCount
    local retCode
    local idx
    local musicalPart = {}    
    local musicalSinger = {}
    local msg
    local tempo = {}
    local tempoList = {}
    local tempoNum
    local timeSig = {}
    local timeSigList = {}
    local timeSigNum
    local seqname = {} -- シーケンスネーム
    local partname -- パートネーム
    local lyric -- 歌詞
    local bpm -- BPM
    local min -- 秒数

-- Musicalパート情報を取得
    retCode, musicalPart, note = VSGetMusicalPart()
    partname = musicalPart.name
    
-- パートの中のノートを取得してノートイベント配列へ格納
    VSSeekToBeginNote()
    idx = 1
    retCode, noteEx = VSGetNextNoteEx()
    while (retCode == 1) do
        noteExList[idx] = noteEx
        retCode, noteEx = VSGetNextNoteEx()
        idx = idx + 1
    end

-- 選択したパートに入っているノートを読み込み
    noteCount = table.getn(noteExList)
-- ノート = 0のエラー処理
    if (noteCount == 0) then
        VSMessageBox("ノートを含むパートを選択してください.", 0)
        return 0
    end

-- 拍子の取得
    VSSeekToBeginTimeSig()
    idx = 1
    retCode, timeSig = VSGetNextTimeSig()
    while (retCode == 1) do
        timeSigList[idx] = timeSig
        retCode, timeSig = VSGetNextTimeSig()
        idx = idx + 1
    end
    timeSigNum = table.getn(timeSigList)

-- テンポの取得
    VSSeekToBeginTempo()
    idx = 1
    retCode, tempo = VSGetNextTempo()
    while (retCode == 1) do
        tempoList[idx] = tempo
        retCode, tempo = VSGetNextTempo()
        idx = idx + 1
    end
    tempoNum = table.getn(tempoList)
    bpm = tempoList[tempoNum].tempo

-- ファイルパス取得
    local sequencePath = VSGetSequencePath()

-- パラメータ入力ダイアログのウィンドウタイトルを設定
    VSDlgSetDialogTitle("名前を付けて保存(ファイル名.txtの保存)")
    local dlgStatus  = VSDlgAddField(field)
    local field = {}
    field.name = "fname"
    field.caption = "ファイル名 (既存のファイルは上書きされます)"
    field.initialVal = partname .. "_VOCALOID3LyricData"
    field.type = 3
    dlgStatus = VSDlgAddField(field)

-- パラメータ入力ダイアログを表示
    dlgStatus = VSDlgDoModal()
    if  (dlgStatus ~= 1) then
    -- OKボタンが押されなかったら終了
        return 0
    end

    local filename
    dlgStatus, filename = VSDlgGetStringValue("fname")

-- txt出力
    io.output(sequencePath .. "_" .. filename .. ".txt")

-- マーカー記入
    for idx = 1, noteCount do
        lyric = noteExList[idx].lyric
        io.write( lyric )
    end
    io.write( "\n" )

-- 正常終了.
    return 0

end
